@@ -26,7 +26,10 @@
 #include <iostream>
 #include <vector>
 #include <map>
+#include <boost/foreach.hpp>
 #include <boost/tokenizer.hpp>
+#include <boost/unordered_map.hpp>
+#include <boost/functional/hash.hpp>
 #include <getopt.h>
 #include <ctime>
 #include <sys/types.h>
@@ -60,7 +63,6 @@
 #   include <windows.h>
 #endif
 
-#include "hashtable.h"
 #include "encoding.h"
 #include "header.h"
 #include "dic_internals.h"
@@ -68,15 +70,21 @@
 
 using namespace std;
 
-//#define DEBUG_LIST
 //#define DEBUG_OUTPUT
-//#define DEBUG_OUTPUT_L2
 #define CHECK_RECURSION
 
 
+unsigned int getFileSize(const string &iFileName)
+{
+    struct stat stat_buf;
+    if (stat(iFileName.c_str(), &stat_buf) < 0)
+        throw DicException(_("Cannot stat file ") + iFileName);
+    return (unsigned int)stat_buf.st_size;
+}
+
 const wchar_t* load_uncompressed(const string &iFileName, unsigned int &ioDicSize)
 {
-    ifstream file(iFileName.c_str());
+    ifstream file(iFileName.c_str(), ios::in | ios::binary);
     if (!file.is_open())
         throw DicException("Could not open file " + iFileName);
 
@@ -89,12 +97,11 @@ const wchar_t* load_uncompressed(const string &iFileName, unsigned int &ioDicSiz
     // Buffer for the wide characters (it will use at most as many characters
     // as the utf-8 version)
     wchar_t *wideBuf = new wchar_t[ioDicSize];
-    unsigned int number;
 
     try
     {
-        number = readFromUTF8(wideBuf, ioDicSize, &buffer.front(),
-                              ioDicSize, "load_uncompressed");
+        unsigned int number = readFromUTF8(wideBuf, ioDicSize, &buffer.front(),
+                                           ioDicSize, "load_uncompressed");
         ioDicSize = number;
         return wideBuf;
     }
@@ -107,14 +114,16 @@ const wchar_t* load_uncompressed(const string &iFileName, unsigned int &ioDicSiz
 }
 
 
-void readLetters(const char *iFileName, DictHeaderInfo &ioHeaderInfo)
+void readLetters(const string &iFileName, DictHeaderInfo &ioHeaderInfo)
 {
-    ifstream in(iFileName);
+    ifstream in(iFileName.c_str());
     if (!in.is_open())
         throw DicException("Could not open file " + string(iFileName));
 
     // Use a more friendly type name
-    typedef boost::tokenizer<boost::char_separator<char> > Tokenizer;
+    typedef boost::tokenizer<boost::char_separator<wchar_t>,
+            std::wstring::const_iterator,
+            std::wstring> Tokenizer;
 
     int lineNb = 1;
     string line;
@@ -124,32 +133,25 @@ void readLetters(const char *iFileName, DictHeaderInfo &ioHeaderInfo)
         if (line == "" || line == "\r" || line == "\n")
             continue;
 
+        // Convert the line to a wstring
+        const wstring &wline = readFromUTF8(line.c_str(), line.size(), "readLetters (1)");
         // Split the lines on space characters
-        vector<string> tokens;
-        boost::char_separator<char> sep(" ");
-        Tokenizer tok(line, sep);
+        boost::char_separator<wchar_t> sep(L" ");
+        Tokenizer tok(wline, sep);
         Tokenizer::iterator it;
-        for (it = tok.begin(); it != tok.end(); ++it)
-        {
-            tokens.push_back(*it);
-        }
+        vector<wstring> tokens(tok.begin(), tok.end());
 
-        // We expect 5 fields on the line, and the first one is a letter, so
-        // it cannot exceed 4 bytes
-        if (tokens.size() != 5 || tokens[0].size() > 4)
+        // We expect at least 5 fields on the line
+        if (tokens.size() < 5)
         {
             ostringstream ss;
-            ss << "readLetters: Invalid line in " << iFileName;
+            ss << "readLetters: Not enough fields in " << iFileName;
             ss << " (line " << lineNb << ")";
             throw DicException(ss.str());
         }
 
-#define MAX_SIZE 4
-        char buff[MAX_SIZE];
-        strncpy(buff, tokens[0].c_str(), MAX_SIZE);
-
-        wstring letter = readFromUTF8(buff, tokens[0].size(), "readLetters");
-
+        // The first field is a single character
+        wstring letter = tokens[0];
         if (letter.size() != 1)
         {
             // On the first line, there could be the BOM...
@@ -165,17 +167,36 @@ void readLetters(const char *iFileName, DictHeaderInfo &ioHeaderInfo)
             {
                 ostringstream ss;
                 ss << "readLetters: Invalid letter at line " << lineNb;
+                ss << " (only one character allowed)";
                 throw DicException(ss.str());
             }
         }
-#undef MAX_SIZE
 
-        ioHeaderInfo.letters += towupper(letter[0]);
+        wchar_t upChar = towupper(letter[0]);
+        ioHeaderInfo.letters += upChar;
 
-        ioHeaderInfo.points.push_back(atoi(tokens[1].c_str()));
-        ioHeaderInfo.frequency.push_back(atoi(tokens[2].c_str()));
-        ioHeaderInfo.vowels.push_back(atoi(tokens[3].c_str()));
-        ioHeaderInfo.consonants.push_back(atoi(tokens[4].c_str()));
+        ioHeaderInfo.points.push_back(_wtoi(tokens[1].c_str()));
+        ioHeaderInfo.frequency.push_back(_wtoi(tokens[2].c_str()));
+        ioHeaderInfo.vowels.push_back(_wtoi(tokens[3].c_str()));
+        ioHeaderInfo.consonants.push_back(_wtoi(tokens[4].c_str()));
+
+        if (tokens.size() > 5)
+        {
+            vector<wstring> inputs(tokens.begin() + 5, tokens.end());
+            // Ensure the input strings are in upper case
+            BOOST_FOREACH(wstring &str, inputs)
+            {
+                std::transform(str.begin(), str.end(), str.begin(), towupper);
+            }
+
+            // If the display stirng is identical to the internal char and if
+            // there is no other input, no need to save this information, as
+            // it is already the default.
+            if (inputs.size() != 1 || inputs[0] != wstring(1, upChar))
+            {
+                ioHeaderInfo.displayInputData[upChar] = inputs;
+            }
+        }
 
         ++lineNb;
     }
@@ -213,7 +234,7 @@ void fix_header(ostream &outfile, DictHeaderInfo &ioHeaderInfo)
 }
 
 
-// Change endianness of the pointes edges, and write them to the given ostream
+// Change endianness of the pointed edges, and write them to the given ostream
 void write_node(uint32_t *ioEdges, unsigned int num, ostream &outfile)
 {
     // Handle endianness
@@ -226,11 +247,6 @@ void write_node(uint32_t *ioEdges, unsigned int num, ostream &outfile)
     printf("writing %d edges\n", num);
     for (int i = 0; i < num; i++)
     {
-#ifdef DEBUG_OUTPUT_L2
-        printf("ptr=%2d t=%d l=%d chr=%2d (%c)\n",
-               ioEdges[i].ptr, ioEdges[i].term, ioEdges[i].last,
-               ioEdges[i].chr, ioEdges[i].chr -1 +'a');
-#endif
         outfile.write((char*)(ioEdges + i), sizeof(DicEdge));
     }
 #else
@@ -247,15 +263,13 @@ void write_node(uint32_t *ioEdges, unsigned int num, ostream &outfile)
 
 // Hashing function for a vector of DicEdge, based on the hashing function
 // of the HashTable
-struct HashVector
+size_t hash_value(const DicEdge &iEdge)
 {
-    unsigned int operator()(const vector<DicEdge> &iKey) const
-    {
-        if (iKey.empty())
-            return 0;
-        return HashPtr(&iKey.front(), iKey.size() * sizeof(DicEdge));
-    }
-};
+    const uint32_t *num = reinterpret_cast<const uint32_t*>(&iEdge);
+    size_t seed = 0;
+    boost::hash_combine(seed, *num);
+    return seed;
+}
 
 #ifdef CHECK_RECURSION
 class IncDec
@@ -279,8 +293,10 @@ int current_rec = 0;
 int max_rec = 0;
 #endif
 
+typedef boost::unordered_map<vector<DicEdge>, unsigned int> HashMap;
+
 /* global variables */
-HashTable<vector<DicEdge>, unsigned int, HashVector> *global_hashtable;
+HashMap global_hashmap;
 
 wchar_t  global_stringbuf[MAX_STRING_LENGTH]; /* Space for current string */
 wchar_t* global_endstring;                    /* Marks END of current string */
@@ -330,7 +346,19 @@ unsigned int makenode(const wchar_t *iPrefix, ostream &outfile,
         newEdge.ptr  = 0;
         newEdge.term = 0;
         newEdge.last = 0;
-        newEdge.chr = iHeader.getCodeFromChar(*global_endstring++ = *global_input++);
+        try
+        {
+            newEdge.chr = iHeader.getCodeFromChar(*global_endstring++ = *global_input++);
+        }
+        catch (DicException &e)
+        {
+            // If an invalid character is found, be specific about the problem
+            ostringstream oss;
+            oss << "Error on line " << 1 + ioHeaderInfo.nwords
+                << ", col " << global_endstring - global_stringbuf
+                << ": " << e.what() << endl;
+            throw DicException(oss.str());
+        }
         edges.push_back(newEdge);
 
         // End of a word?
@@ -373,18 +401,18 @@ unsigned int makenode(const wchar_t *iPrefix, ostream &outfile,
     // Mark the last edge
     edges.back().last = 1;
 
-    const unsigned int *saved_position = global_hashtable->find(edges);
-    if (saved_position)
+    HashMap::const_iterator itMap = global_hashmap.find(edges);
+    if (itMap != global_hashmap.end())
     {
         ioHeaderInfo.edgessaved += numedges;
         ioHeaderInfo.nodessaved++;
 
-        return *saved_position;
+        return itMap->second;
     }
     else
     {
         unsigned int node_pos = ioHeaderInfo.edgesused;
-        global_hashtable->add(edges, ioHeaderInfo.edgesused);
+        global_hashmap[edges] = ioHeaderInfo.edgesused;
         ioHeaderInfo.edgesused += numedges;
         ioHeaderInfo.nodesused++;
         write_node(reinterpret_cast<uint32_t*>(&edges.front()),
@@ -410,18 +438,31 @@ void printUsage(const string &iBinaryName)
          << "  " << iBinaryName << _(" -d 'ODS 5.0' -l letters.txt -i ods5.txt -o ods5.dawg") << endl
          << endl
          << _("The file containing the letters (--letters switch) must be UTF-8 encoded.") << endl
-         << _("Each line corresponds to one letter, and must contain 5 fields separated with ") << endl
-         << _("one or more space(s).") << endl
-         << _(" - 1st field: the letter itself") << endl
+         << _("Each line corresponds to one letter, and must contain at least 5 fields separated with "
+              "one or more space(s).") << endl
+         << _(" - 1st field: the letter itself, as stored in the input file (single character)") << endl
          << _(" - 2nd field: the points of the letter") << endl
          << _(" - 3rd field: the frequency of the letter (how many letters of this kind in the game)") << endl
          << _(" - 4th field: 1 if the letter is considered as a vowel in Scrabble game, 0 otherwise") << endl
          << _(" - 5th field: 1 if the letter is considered as a consonant in Scrabble game, 0 otherwise") << endl
+         << _(" - 6th field (optional): display string for the letter (default: the letter itself)") << endl
+         << _(" - other fields (optional): input strings for the letter, in addition to the display string") << endl
+         << endl
          << _("Example for french:") << endl
-         << _("A 1 9 1 0") << endl
-         << _("[...]") << endl
-         << _("Z 10 1 0 1") << endl
-         << _("? 0 2 1 1") << endl;
+         << "A 1 9 1 0" << endl
+         << "[...]" << endl
+         << "Z 10 1 0 1" << endl
+         << "? 0 2 1 1" << endl
+         << endl
+         << _("Example for catalan:") << endl
+         << "A 1 12 1 0" << endl
+         << "[...]" << endl
+         // TRANSLATORS: the first "L.L" must be translated "L·L",
+         // and the last one translated "ĿL"
+         << _("W 10 1 0 1 L.L L.L L-L L.L") << endl
+         << "X 10 1 0 1" << endl
+         << "Y 10 1 0 1 NY" << endl
+         << "[...]" << endl;
 }
 
 
@@ -507,13 +548,7 @@ int main(int argc, char* argv[])
             exit(1);
         }
 
-        struct stat stat_buf;
-        if (stat(inFileName.c_str(), &stat_buf) < 0)
-        {
-            cerr << _("Cannot stat uncompressed dictionary ") << inFileName << endl;
-            exit(1);
-        }
-        unsigned int dicsize = (unsigned int)stat_buf.st_size;
+        unsigned int dicSize = getFileSize(inFileName);
 
         ofstream outfile(outFileName.c_str(), ios::out | ios::binary | ios::trunc);
         if (!outfile.is_open())
@@ -524,15 +559,11 @@ int main(int argc, char* argv[])
 
         clock_t startLoadTime = clock();
         // FIXME: not exception safe
-        const wchar_t *uncompressed = load_uncompressed(inFileName, dicsize);
+        const wchar_t *uncompressed = load_uncompressed(inFileName, dicSize);
         clock_t endLoadTime = clock();
 
         global_input = uncompressed;
-        global_endofinput = global_input + dicsize;
-
-#define SCALE 0.6
-        global_hashtable = new HashTable<vector<DicEdge>, unsigned int, HashVector>((unsigned int)(dicsize * SCALE));
-#undef SCALE
+        global_endofinput = global_input + dicSize;
 
         headerInfo.dawg = true;
         Header tempHeader = skip_init_header(outfile, headerInfo);
@@ -562,7 +593,6 @@ int main(int argc, char* argv[])
         Header aHeader(headerInfo);
         aHeader.print();
 
-        delete global_hashtable;
         delete[] uncompressed;
         outfile.close();
 
@@ -578,6 +608,5 @@ int main(int argc, char* argv[])
         cerr << e.what() << endl;
         return 1;
     }
-
 }
 
