@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2002-2007 Antoine Fraboulet & Olivier Teulière
+ * Copyright (C) 2002-2009 Antoine Fraboulet & Olivier Teulière
  * Authors: Antoine Fraboulet <antoine.fraboulet @@ free.fr>
  *          Olivier Teulière <ipkiss @@ gmail.com>
  *
@@ -44,6 +44,18 @@ struct params_7plus1_t;
 struct params_regexp_t;
 class DicEdge;
 
+/**
+ * A wdstring is a display string, i.e. it can contain more chars than
+ * the represented string. The difference arises in languages such as Catalan,
+ * where for example "QU" is made of 2 real characters, but corresponds to a
+ * single tile.
+ *
+ * The wdstring type has no particular interest other than signaling
+ * a bit more precisely the type of contents of the string.
+ */
+typedef wstring wdstring;
+typedef wstring wistring;
+
 class Dictionary
 {
 public:
@@ -82,6 +94,30 @@ public:
     bool validateLetters(const wstring &iLetters,
                          const wstring &iAccepted = L"") const;
 
+    /**
+     * Check whether all the given characters can be part of an input
+     * form for one or more dictionary letter, or are one of the other
+     * accepted letters.
+     * Return true if this is the case, false otherwise
+     */
+    bool validateInputChars(const wstring &iLetters,
+                            const wstring &iAccepted = L"") const;
+
+    /**
+     * Convert the given string (made of internal characters)
+     * into a string suitable for display
+     */
+    wdstring convertToDisplay(const wstring &iWord) const;
+
+    /**
+     * Convert the given string (direct user input)
+     * into a string suitable for internal use in Eliot.
+     * For example, in Catalan, it will convert the L.L substring
+     * into the W character (because it is the internal character
+     * associated to the input string "L.L").
+     */
+    wstring convertFromInput(const wistring &iWord) const;
+
     /** Return a vector containing one of each possible tile */
     const vector<Tile>& getAllTiles() const { return m_tilesVect; }
 
@@ -182,7 +218,7 @@ public:
      * @param oWordList: results
      * @param iMaxResults: maximum number of returned results (0 means no limit)
      */
-    void searchBenj(const wstring &iWord, vector<wstring> &oWordList,
+    void searchBenj(const wstring &iWord, vector<wdstring> &oWordList,
                     unsigned int iMaxResults = 0) const;
 
     /**
@@ -191,27 +227,23 @@ public:
      * @param oWordList: results
      * @param iMaxResults: maximum number of returned results (0 means no limit)
      */
-    void searchRacc(const wstring &iWord, vector<wstring> &oWordList,
+    void searchRacc(const wstring &iWord, vector<wdstring> &oWordList,
                     unsigned int iMaxResults = 0) const;
 
-    /**
-     * Search for crosswords
-     * @param iMask: letters
-     * @param oWordList: results
-     * @param iMaxResults: maximum number of returned results (0 means no limit)
-     */
-    void searchCross(const wstring &iMask, vector<wstring> &oWordList,
-                     unsigned int iMaxResults = 0) const;
-
     /**
      * Search for all feasible word with "rack" plus one letter
+     * XXX: the key in the map is the internal code, because it allows an easy
+     * iteration in the map in the order of the dictionary letters.
+     * Maybe a more powerful structure should be provided, to hide the internal
+     * chars to the caller.
+     *
      * @param iRack: letters
-     * @param oWordlist: results
+     * @param oWordlist: results (grouped by code of the added character)
      * @param joker: true if the search must be performed when a joker is in the rack
      * @param iMaxResults: maximum number of returned results (0 means no limit)
      */
     void search7pl1(const wstring &iRack,
-                    map<wchar_t, vector<wstring> > &oWordList,
+                    map<unsigned int, vector<wdstring> > &oWordList,
                     bool joker) const;
 
     /**
@@ -226,7 +258,7 @@ public:
      * @throw InvalidRegexpException When the regular expression cannot be parsed
      */
     bool searchRegExp(const wstring &iRegexp,
-                      vector<wstring> &oWordList,
+                      vector<wdstring> &oWordList,
                       unsigned int iMinLength,
                       unsigned int iMaxLength,
                       unsigned int iMaxResults = 0) const;
@@ -239,12 +271,40 @@ private:
     Header *m_header;
     uint32_t *m_dawg;
 
-    /** Letters of the dictionary, both in uppercase and lowercase */
+    /**
+     * Letters of the dictionary, both in uppercase and lowercase
+     * (internal representation)
+     */
     wstring m_allLetters;
 
+    /**
+     * All the possible characters possibly used to input letters of
+     * the dictionary, both in uppercase and lowercase
+     */
+    wstring m_allInputChars;
+
     /// Vector of available tiles
     vector<Tile> m_tilesVect;
 
+    /**
+     * Associate to some internal chars (both the lower case and
+     * upper case versions) all the corresponding input strings.
+     * The first one is always the display string.
+     *
+     * Note: only the chars which have more than 1 input string,
+     * or which have a display string different from the internal char,
+     * are present in the map.
+     */
+    map<wchar_t, vector<wstring> > m_displayInputCache;
+
+    /**
+     * True if at least one display strings is different from the internal
+     * char, false otherwise. This flag is more precise than checking the size
+     * of m_displayInputCache, because a tile can have input strings even if
+     * its display string is equal to the internal char.
+     */
+    bool m_hasDisplay;
+
     static const Dictionary *m_dic;
 
     void convertDataToArch();
@@ -257,22 +317,18 @@ private:
      */
     const DicEdge * seekEdgePtr(const wchar_t *s, const DicEdge *eptr) const;
 
-    /// Helper for searchCross()
-    void searchCrossRec(struct params_cross_t *params,
-                        vector<wstring> &oWordList,
-                        const DicEdge *edgeptr,
-                        unsigned int iMaxResults) const;
-
     /// Helper for search7pl1()
-    void searchWordByLen(struct params_7plus1_t *params,
+    void searchWordByLen(struct params_7plus1_t &params,
                          int i, const DicEdge *edgeptr) const;
 
     /// Helper for searchRegExp()
-    void searchRegexpRec(struct params_regexp_t *params,
+    void searchRegexpRec(const struct params_regexp_t &params,
                          int state,
                          const DicEdge *edgeptr,
-                         vector<wstring> &oWordList,
-                         unsigned int iMaxResults) const;
+                         vector<wdstring> &oWordList,
+                         unsigned int iMaxResults,
+                         const wdstring &iCurrWord = L"",
+                         unsigned int iNbChars = 0) const;
 };
 
 #endif /* _DIC_H_ */
