@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2007 Olivier Teulière
+ * Copyright (C) 2007-2009 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,10 @@
 
 using namespace std;
 
+// XXX: duplicated typedef (also present in dic.h)
+typedef wstring wdstring;
+typedef wstring wistring;
+
 
 /**
  * Structure used to create a Header object.
@@ -49,6 +53,7 @@ struct DictHeaderInfo
     vector<uint8_t> frequency;
     vector<bool> vowels;
     vector<bool> consonants;
+    map<wchar_t, vector<wstring> > displayInputData;
 };
 
 
@@ -105,12 +110,15 @@ public:
     wstring      getName()         const { return m_dicName; }
     DictType     getType()         const { return m_type; }
     wstring      getLetters()      const { return m_letters; }
+    wstring      getInputChars()   const { return m_inputChars; }
     uint8_t      getPoints(unsigned int iCode) const { return m_points[iCode - 1]; }
     uint8_t      getFrequency(unsigned int iCode) const { return m_frequency[iCode - 1]; }
     bool         isVowel(unsigned int iCode) const { return m_vowels[iCode - 1]; }
     bool         isConsonant(unsigned int iCode) const { return m_consonants[iCode - 1]; }
     //@}
 
+    const map<wchar_t, vector<wstring> > & getDisplayInputData() const { return m_displayAndInputData; }
+
     /**
      * Return the letter corresponding to the given code
      */
@@ -121,6 +129,16 @@ public:
      */
     unsigned int getCodeFromChar(wchar_t iChar) const;
 
+    /**
+     * Return the display string corresponding to the given code
+     */
+    const wdstring & getDisplayStr(unsigned int iCode) const;
+
+    /**
+     * Return all the accepted input strings corresponding to the given code
+     */
+    vector<wistring> getInputStr(unsigned int iCode) const;
+
     /**
      * Print a readable summary of the header on standard output
      */
@@ -153,9 +171,12 @@ private:
     /// Dictionary name (e.g.: ODS 5.0)
     wstring m_dicName;
 
-    /// The letters constituting the words of the dictionary
+    /// (Internal) letters constituting the words of the dictionary
     wstring m_letters;
 
+    /// Characters usable to input the dictionary letters
+    wstring m_inputChars;
+
     /// Points of the letters
     vector<uint8_t> m_points;
 
@@ -168,8 +189,15 @@ private:
     /// Consonants
     vector<bool> m_consonants;
 
+    /// Additional display and input strings for some letters
+    map<wchar_t, vector<wstring> > m_displayAndInputData;
+
+    /// Cache for the char --> code associations
     map<wchar_t, unsigned int> m_mapCodeFromChar;
 
+    /// Cache for the display string of each code
+    vector<wdstring> m_displayCache;
+
     /**
      * Load the header from a file
      * @param iStream: Input stream where to read the header
@@ -177,8 +205,20 @@ private:
      */
     void read(istream &iStream);
 
-    /** Build m_mapCodeFromChar */
-    void buildMapCodeFromChar();
+    /** Build various caches */
+    void buildCaches();
+
+    /**
+     * Fill the m_displayAndInputData field from the serialized data
+     * of the given string
+     */
+    void readDisplayAndInput(const wstring &serialized);
+
+    /**
+     * Return a serialized version of the data contained in the
+     * m_displayAndInputData field
+     */
+    wstring writeDisplayAndInput() const;
 };
 
 #endif /* _HEADER_H */
