@@ -21,6 +21,7 @@
 
 #include <sstream>
 #include <string>
+#include <algorithm>
 #include <wctype.h>
 #include "tile.h"
 #include "header.h"
@@ -95,6 +96,29 @@ unsigned int Tile::getPoints() const
 }
 
 
+wstring Tile::getDisplayStr() const
+{
+    if (m_code == 0)
+        throw DicException("Tile::getDisplayStr: Invalid tile");
+    if (m_joker && iswalpha(m_char))
+    {
+        wstring str = m_header->getDisplayStr(m_code);
+        std::transform(str.begin(), str.end(), str.begin(), towlower);
+        return str;
+    }
+    return m_header->getDisplayStr(m_code);
+}
+
+
+vector<wistring> Tile::getInputStr() const
+{
+    if (m_code == 0)
+        throw DicException("Tile::getInputStr: Invalid tile");
+    // We always return uppercase strings
+    return m_header->getInputStr(m_code);
+}
+
+
 wchar_t Tile::toChar() const
 {
     if (m_joker)
