@@ -81,7 +81,7 @@ unsigned int Bag::getNbConsonants() const
 void Bag::takeTile(const Tile &iTile)
 {
     ASSERT(in(iTile),
-           "The bag does not contain the letter " + convertToMb(iTile.toChar()));
+           "The bag does not contain the letter " + convertToMb(iTile.getDisplayStr()));
 
     m_tilesMap[iTile]--;
     m_ntiles--;
@@ -91,7 +91,7 @@ void Bag::takeTile(const Tile &iTile)
 void Bag::replaceTile(const Tile &iTile)
 {
     ASSERT(in(iTile) < iTile.maxNumber(),
-           "Cannot replace tile: " + convertToMb(iTile.toChar()));
+           "Cannot replace tile: " + convertToMb(iTile.getDisplayStr()));
 
     m_tilesMap[iTile]++;
     m_ntiles++;
