@@ -20,6 +20,7 @@
  *****************************************************************************/
 
 #include <wctype.h>
+#include <algorithm>
 
 #include "dic.h"
 
@@ -94,7 +95,7 @@ Board::Board():
     m_crossCol(BOARD_REALDIM, Cross()),
     m_pointRow(BOARD_REALDIM, -1),
     m_pointCol(BOARD_REALDIM, -1),
-    m_testsRow(BOARD_REALDIM, 0),
+    m_testsRow(BOARD_REALDIM, false),
     m_isEmpty(true)
 {
     // No cross check allowed around the board
@@ -112,33 +113,23 @@ Board::Board():
 }
 
 
-wchar_t Board::getChar(int iRow, int iCol) const
-{
-    wchar_t letter = 0;
-    Tile tile = getTile(iRow, iCol);
-    if (!tile.isEmpty())
-    {
-        letter = tile.toChar();
-        if (isJoker(iRow, iCol))
-            letter = towlower(letter);
-    }
-    return letter;
-}
-
-int Board::getCharAttr(int iRow, int iCol) const
-{
-    int t = getTestChar(iRow, iCol);
-    int j = isJoker(iRow, iCol);
-    return  (t << 1) | j;
-}
-
-
 const Tile& Board::getTile(int iRow, int iCol) const
 {
     return m_tilesRow[iRow][iCol];
 }
 
 
+wstring Board::getDisplayStr(int iRow, int iCol) const
+{
+    ASSERT(!isVacant(iRow, iCol),
+           "Trying to get the display string on an empty board square");
+    wstring str = getTile(iRow, iCol).getDisplayStr();
+    if (isJoker(iRow, iCol))
+        std::transform(str.begin(), str.end(), str.begin(), towlower);
+    return str;
+}
+
+
 bool Board::isJoker(int iRow, int iCol) const
 {
     return m_jokerRow[iRow][iCol];
@@ -353,8 +344,12 @@ int Board::checkRoundAux(const Matrix<Tile> &iTilesMx,
     if (isolated && !m_isEmpty)
         return 5;
     // The first word must be horizontal
+    // Deactivated, as a vertical first word is allowed in free games,
+    // and possibly in duplicate games as well (it depends on the sources)
+#if 0
     if (m_isEmpty && iRound.getCoord().getDir() == Coord::VERTICAL)
         return 6;
+#endif
     // The first word must cover the H8 square
     if (m_isEmpty
         && (row != 8 || col > 8 || col + iRound.getWordLen() <= 8))
@@ -410,7 +405,7 @@ void Board::testRound(const Round &iRound)
                 t = iRound.getTile(i);
                 m_tilesRow[row][col + i] = t;
                 m_jokerRow[row][col + i] = iRound.isJoker(i);
-                m_testsRow[row][col + i] = 1;
+                m_testsRow[row][col + i] = true;
 
                 m_tilesCol[col + i][row] = t;
                 m_jokerCol[col + i][row] = iRound.isJoker(i);
@@ -426,7 +421,7 @@ void Board::testRound(const Round &iRound)
                 t = iRound.getTile(i);
                 m_tilesRow[row + i][col] = t;
                 m_jokerRow[row + i][col] = iRound.isJoker(i);
-                m_testsRow[row + i][col] = 1;
+                m_testsRow[row + i][col] = true;
 
                 m_tilesCol[col][row + i] = t;
                 m_jokerCol[col][row + i] = iRound.isJoker(i);
@@ -445,7 +440,7 @@ void Board::removeTestRound()
             if (m_testsRow[row][col])
             {
                 m_tilesRow[row][col] = Tile();
-                m_testsRow[row][col] = 0;
+                m_testsRow[row][col] = false;
                 m_jokerRow[row][col] = false;
 
                 m_tilesCol[col][row] = Tile();
@@ -456,7 +451,7 @@ void Board::removeTestRound()
 }
 
 
-char Board::getTestChar(int iRow, int iCol) const
+bool Board::isTestChar(int iRow, int iCol) const
 {
     return m_testsRow[iRow][iCol];
 }
