@@ -28,6 +28,11 @@
 
 #ifdef DEBUG
 #   include <iostream>
+#   include <cstdlib>
+
+using std::cerr;
+using std::endl;
+
 // Assertion macro: if the condition is not verified, print a message on stderr
 // and stops execution, otherwise do nothing.
 #   define ASSERT(cond, msg) \
@@ -45,9 +50,3 @@
 
 #endif
 
-/// Local Variables:
-/// mode: c++
-/// mode: hs-minor
-/// c-basic-offset: 4
-/// indent-tabs-mode: nil
-/// End:
