@@ -1,6 +1,6 @@
 /*****************************************************************************
  * Eliot
- * Copyright (C) 2008 Olivier Teulière
+ * Copyright (C) 2008-2009 Olivier Teulière
  * Authors: Olivier Teulière <ipkiss @@ gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 
 #include <map>
 #include <vector>
+#include <algorithm>
 #include <QtGui/QTreeView>
 #include <QtGui/QStandardItemModel>
 #include <QtGui/QVBoxLayout>
@@ -73,6 +74,8 @@ DicToolsWidget::DicToolsWidget(QWidget *parent)
 
     redPalette = labelCheck->palette();
     redPalette.setColor(QPalette::Foreground, Qt::red);
+    darkYellowPalette = labelCheck->palette();
+    darkYellowPalette.setColor(QPalette::Foreground, Qt::darkYellow);
     greenPalette = labelCheck->palette();
     greenPalette.setColor(QPalette::Foreground, Qt::darkGreen);
 
@@ -96,12 +99,13 @@ DicToolsWidget::DicToolsWidget(QWidget *parent)
 
     m_dicInfoModel = new QStandardItemModel(this);
     treeViewDicLetters->setModel(m_dicInfoModel);
-    m_dicInfoModel->setColumnCount(5);
+    m_dicInfoModel->setColumnCount(6);
     m_dicInfoModel->setHeaderData(0, Qt::Horizontal, _q("Letter"), Qt::DisplayRole);
     m_dicInfoModel->setHeaderData(1, Qt::Horizontal, _q("Points"), Qt::DisplayRole);
     m_dicInfoModel->setHeaderData(2, Qt::Horizontal, _q("Frequency"), Qt::DisplayRole);
     m_dicInfoModel->setHeaderData(3, Qt::Horizontal, _q("Vowel?"), Qt::DisplayRole);
     m_dicInfoModel->setHeaderData(4, Qt::Horizontal, _q("Consonant?"), Qt::DisplayRole);
+    m_dicInfoModel->setHeaderData(5, Qt::Horizontal, _q("Alternative inputs"), Qt::DisplayRole);
 }
 
 
@@ -147,15 +151,26 @@ void DicToolsWidget::refreshCheck()
         labelCheck->setText("");
     else
     {
-        bool res = m_dic->searchWord(qtw(rack->text()));
+        if (!rack->hasAcceptableInput())
+        {
+            labelCheck->setText(_q("Invalid or incomplete letters"));
+            labelCheck->setPalette(darkYellowPalette);
+            return;
+        }
+
+        wstring input = m_dic->convertFromInput(qtw(rack->text()));
+        bool res = m_dic->searchWord(input);
+        // Convert the input to uppercase
+        std::transform(input.begin(), input.end(), input.begin(), towupper);
+        const wdstring &dispStr = m_dic->convertToDisplay(input);
         if (res)
         {
-            labelCheck->setText(_q("The word '%1' exists").arg(rack->text().toUpper()));
+            labelCheck->setText(_q("The word '%1' exists").arg(qfw(dispStr)));
             labelCheck->setPalette(greenPalette);
         }
         else
         {
-            labelCheck->setText(_q("The word '%1' does not exist").arg(rack->text().toUpper()));
+            labelCheck->setText(_q("The word '%1' does not exist").arg(qfw(dispStr)));
             labelCheck->setPalette(redPalette);
         }
     }
@@ -172,31 +187,31 @@ void DicToolsWidget::refreshPlus1()
     if (m_dic == NULL)
     {
         model->setHeaderData(0, Qt::Horizontal,
-                               _q("Please select a dictionary"),
-                               Qt::DisplayRole);
+                             _q("Please select a dictionary"),
+                             Qt::DisplayRole);
         return;
     }
-    else
-    {
-        model->setHeaderData(0, Qt::Horizontal,
-                               _q("Rack: %1").arg(rack->text().toUpper()),
-                               Qt::DisplayRole);
-    }
 
-    if (rack->text() != "")
+    const wstring &input = m_dic->convertFromInput(qtw(rack->text().toUpper()));
+    const wdstring &disp = m_dic->convertToDisplay(input);
+    model->setHeaderData(0, Qt::Horizontal,
+                         _q("Rack: %1").arg(qfw(disp)),
+                         Qt::DisplayRole);
+
+    if (input != L"")
     {
-        map<wchar_t, vector<wstring> > wordList;
-        m_dic->search7pl1(qtw(rack->text()), wordList, true);
+        map<unsigned int, vector<wstring> > wordList;
+        m_dic->search7pl1(input, wordList, true);
 
         int rowNum = 0;
-        map<wchar_t, vector<wstring> >::const_iterator it;
+        map<unsigned int, vector<wstring> >::const_iterator it;
         for (it = wordList.begin(); it != wordList.end(); it++)
         {
             // Create the header line
             model->insertRow(rowNum);
             const QModelIndex &index = model->index(rowNum, 0);
-            if (it->first)
-                model->setData(index, qfw(wstring(1, it->first)));
+            if (it->first != 0)
+                model->setData(index, qfw(m_dic->getHeader().getDisplayStr(it->first)));
             else
                 model->setData(index, _q("Anagrams"));
             treeView->setExpanded(index, true);
@@ -229,14 +244,14 @@ void DicToolsWidget::refreshRegexp()
                                Qt::DisplayRole);
         return;
     }
-    else
-    {
-        model->setHeaderData(0, Qt::Horizontal,
-                               _q("Regular expression: %1").arg(rack->text().toUpper()),
-                               Qt::DisplayRole);
-    }
 
-    if (rack->text() != "")
+    const wstring &input = m_dic->convertFromInput(qtw(rack->text().toUpper()));
+    const wdstring &disp = m_dic->convertToDisplay(input);
+    model->setHeaderData(0, Qt::Horizontal,
+                         _q("Regular expression: %1").arg(qfw(disp)),
+                         Qt::DisplayRole);
+
+    if (input != L"")
     {
         unsigned lmin = spinBoxMinLength->value();
         unsigned lmax = spinBoxMaxLength->value();
@@ -248,8 +263,7 @@ void DicToolsWidget::refreshRegexp()
         int rowNum = 0;
         try
         {
-            res = m_dic->searchRegExp(qtw(rack->text()), wordList,
-                                      lmin, lmax, limit);
+            res = m_dic->searchRegExp(input, wordList, lmin, lmax, limit);
         }
         catch (InvalidRegexpException &e)
         {
@@ -288,7 +302,7 @@ void DicToolsWidget::refreshDicInfo()
     {
         const Header &header = m_dic->getHeader();
         lineEditName->setText(qfw(header.getName()));
-        lineEditLetters->setText(qfw(header.getLetters()));
+        lineEditLetters->setText(qfw(m_dic->convertToDisplay(header.getLetters())));
         spinBoxWords->setValue(header.getNbWords());
 
         QStandardItemModel *model = m_dicInfoModel;
@@ -300,13 +314,20 @@ void DicToolsWidget::refreshDicInfo()
         {
             model->insertRow(rowNum);
             model->setData(model->index(rowNum, 0),
-                           qfw(wstring(1, it->toChar())));
+                           qfw(it->getDisplayStr()));
             model->setData(model->index(rowNum, 1), it->getPoints());
             model->setData(model->index(rowNum, 2), it->maxNumber());
             model->setData(model->index(rowNum, 3),
                            it->isVowel() ? _q("Yes") : _q("No"));
             model->setData(model->index(rowNum, 4),
                            it->isConsonant() ? _q("Yes") : _q("No"));
+            const vector<wistring> &inputVect = it->getInputStr();
+            wstring tmp;
+            for (unsigned int i = 1; i < inputVect.size(); ++i)
+            {
+                tmp += inputVect[i] + L"  ";
+            }
+            model->setData(model->index(rowNum, 5), qfw(tmp));
             ++rowNum;
         }
         treeViewDicLetters->resizeColumnToContents(0);
@@ -335,12 +356,19 @@ QValidator::State DicRackValidator::validate(QString &input, int &) const
     if (input == "")
         return Intermediate;
 
-    //input = input.toUpper();
-
-    // The string is invalid if it contains characters not present
-    // in the dictionary or if it contains a '?'
-    if (!m_dic->validateLetters(qtw(input)))
+    // The string is invalid if it contains invalid input characters
+    const wistring &winput = qtw(input);
+    if (!m_dic->validateInputChars(winput))
         return Invalid;
+
+    // Convert the string to internal letters
+    const wstring &intInput = m_dic->convertFromInput(winput);
+    // The string is invalid if it contains characters not present
+    // in the dictionary
+    if (!m_dic->validateLetters(intInput))
+        return Intermediate;
+
+    // A '?' may not be acceptable
     if (!m_acceptJoker && input.contains('?'))
         return Invalid;
     // Do not accept more than 2 jokers
@@ -366,18 +394,20 @@ QValidator::State RegexpValidator::validate(QString &input, int &) const
     if (input == "")
         return Intermediate;
 
-    // Strip regular expression characters
-    QString copy(input);
-    QString authorizedChars = ".[]()*+?:^";
-    for (int i = 0; i < authorizedChars.size(); ++i)
-    {
-        copy.remove(authorizedChars[i]);
-    }
+    wstring authorizedChars = L".[]()*+?:^";
 
+    // The string is invalid if it contains invalid input characters
+    const wistring &winput = qtw(input);
+    if (!m_dic->validateInputChars(winput, authorizedChars))
+        return Invalid;
+
+    // Convert the string to internal letters
+    const wstring &intInput = m_dic->convertFromInput(winput);
     // The string is invalid if it contains characters not present
     // in the dictionary
-    if (!m_dic->validateLetters(qtw(copy)))
-        return Invalid;
+    if (!m_dic->validateLetters(intInput, authorizedChars))
+        return Intermediate;
+
     return Acceptable;
 }
 
