@@ -38,6 +38,7 @@
 #endif
 
 #include "dic.h"
+#include "header.h"
 #include "dic_exception.h"
 #include "game_io.h"
 #include "game_factory.h"
@@ -133,6 +134,15 @@ wstring checkAlphaToken(const vector<wstring> &tokens, uint8_t index)
 }
 
 
+wstring checkLettersToken(const vector<wstring> &tokens, uint8_t index,
+                          const Dictionary &iDic)
+{
+    if (tokens.size() <= index)
+        return L"";
+    return iDic.convertFromInput(tokens[index]);
+}
+
+
 wstring checkAlphaNumToken(const vector<wstring> &tokens, uint8_t index)
 {
     if (tokens.size() <= index)
@@ -147,20 +157,6 @@ wstring checkAlphaNumToken(const vector<wstring> &tokens, uint8_t index)
 }
 
 
-wstring checkAlphaPlusJokerToken(const vector<wstring> &tokens, uint8_t index)
-{
-    if (tokens.size() <= index)
-        return L"";
-    const wstring &wstr = tokens[index];
-    BOOST_FOREACH(wchar_t wch, wstr)
-    {
-        if (!iswalpha(wch) && wch != L'+' && wch != L'?')
-            return L"";
-    }
-    return wstr;
-}
-
-
 wstring checkNumToken(const vector<wstring> &tokens, uint8_t index)
 {
     if (tokens.size() <= index)
@@ -375,7 +371,7 @@ void commonCommands(PublicGame &iGame, const vector<wstring> &tokens)
         displayData(iGame, tokens);
     else if (tokens[0][0] == L'd')
     {
-        const wstring &word = checkAlphaToken(tokens, 1);
+        const wstring &word = checkLettersToken(tokens, 1, iGame.getDic());
         if (word == L"")
             helpDuplicate();
         else
@@ -413,7 +409,7 @@ void commonCommands(PublicGame &iGame, const vector<wstring> &tokens)
     }
     else if (tokens[0][0] == L'j')
     {
-        const wstring &word = checkAlphaToken(tokens, 1);
+        const wstring &word = checkLettersToken(tokens, 1, iGame.getDic());
         if (word == L"")
             helpDuplicate();
         else
@@ -485,7 +481,7 @@ void handleRegexp(const Dictionary& iDic, const vector<wstring> &tokens)
     printf("search for %s (%d,%d,%d)\n", convertToMb(regexp).c_str(),
            nres, lmin, lmax);
 
-    vector<wstring> wordList;
+    vector<wdstring> wordList;
     try
     {
         iDic.searchRegExp(regexp, wordList, lmin, lmax, nres);
@@ -496,7 +492,7 @@ void handleRegexp(const Dictionary& iDic, const vector<wstring> &tokens)
         return;
     }
 
-    BOOST_FOREACH(const wstring &wstr, wordList)
+    BOOST_FOREACH(const wdstring &wstr, wordList)
     {
         printf("%s\n", convertToMb(wstr).c_str());
     }
@@ -512,7 +508,7 @@ void setSetting(const vector<wstring> &tokens)
         return;
     }
     const wstring &type = checkAlphaToken(tokens, 1);
-    if (type == L"" || type.size() != 1 ||
+    if (type.size() != 1 ||
         (type[0] != L'b' && type[0] != L'i'))
     {
         printf("Invalid type\n");
@@ -589,7 +585,7 @@ void loopTraining(PublicGame &iGame)
                             helpTraining();
                         else
                         {
-                            const wstring &word = checkAlphaToken(tokens, 2);
+                            const wstring &word = checkLettersToken(tokens, 2, iGame.getDic());
                             if (word == L"")
                                 helpTraining();
                             else
@@ -598,22 +594,22 @@ void loopTraining(PublicGame &iGame)
                                 {
                                     case L'b':
                                         {
-                                            vector<wstring> wordList;
+                                            vector<wdstring> wordList;
                                             iGame.getDic().searchBenj(word, wordList);
-                                            BOOST_FOREACH(const wstring &wstr, wordList)
+                                            BOOST_FOREACH(const wdstring &wstr, wordList)
                                                 cout << convertToMb(wstr) << endl;
                                         }
                                         break;
                                     case L'p':
                                         {
-                                            map<wchar_t, vector<wstring> > wordMap;
+                                            map<unsigned int, vector<wdstring> > wordMap;
                                             iGame.getDic().search7pl1(word, wordMap, false);
-                                            map<wchar_t, vector<wstring> >::const_iterator it;
+                                            map<unsigned int, vector<wdstring> >::const_iterator it;
                                             for (it = wordMap.begin(); it != wordMap.end(); ++it)
                                             {
-                                                if (it->first)
-                                                    cout << "+" << convertToMb(it->first) << endl;
-                                                BOOST_FOREACH(const wstring &wstr, it->second)
+                                                if (it->first != 0)
+                                                    cout << "+" << convertToMb(iGame.getDic().getHeader().getDisplayStr(it->first)) << endl;
+                                                BOOST_FOREACH(const wdstring &wstr, it->second)
                                                 {
                                                     cout << "  " << convertToMb(wstr) << endl;
                                                 }
@@ -622,9 +618,9 @@ void loopTraining(PublicGame &iGame)
                                         break;
                                     case L'r':
                                         {
-                                            vector<wstring> wordList;
+                                            vector<wdstring> wordList;
                                             iGame.getDic().searchRacc(word, wordList);
-                                            BOOST_FOREACH(const wstring &wstr, wordList)
+                                            BOOST_FOREACH(const wdstring &wstr, wordList)
                                                 cout << convertToMb(wstr) << endl;
                                         }
                                         break;
@@ -652,29 +648,14 @@ void loopTraining(PublicGame &iGame)
                     break;
                 case L't':
                     {
-                        const wstring &letters = checkAlphaPlusJokerToken(tokens, 1);
+                        const wstring &letters =
+                            checkLettersToken(tokens, 1, iGame.getDic());
                         if (letters == L"")
                             helpTraining();
                         else
                             iGame.trainingSetRackManual(false, letters);
                     }
                     break;
-                case L'x':
-                    {
-                        const wstring &cross = checkCrossToken(tokens, 1);
-                        if (cross == L"")
-                            helpTraining();
-                        else
-                        {
-                            vector<wstring> wordList;
-                            iGame.getDic().searchCross(cross, wordList);
-                            BOOST_FOREACH(const wstring &wstr, wordList)
-                            {
-                                printf("  %s\n", convertToMb(wstr).c_str());
-                            }
-                        }
-                    }
-                    break;
                 case L'*':
                     iGame.trainingSetRackRandom(false, PublicGame::kRACK_ALL);
                     break;
@@ -744,7 +725,7 @@ void loopFreegame(PublicGame &iGame)
                         /* You can pass your turn without changing any letter */
                         if (tokens.size() > 1)
                         {
-                            letters = checkAlphaToken(tokens, 1);
+                            letters = checkLettersToken(tokens, 1, iGame.getDic());
                             if (letters == L"")
                                 fprintf(stderr, "Invalid letters\n");
                         }
